function FID = CQ2nd(param)
% FID = CQ2nd(param)
% computes spikelet echo tran (QCPMG)  for jumps among sites defined by
% second order quadrupole frequencies
% see the file express.m for parameter definitions
%
% computes only central transition, stationary sample
% no chemical shifts or shift anisotropy
%
% always computes at least one spin echo
%
% functions called by CQ2nd
%           gettiles.m         -- get tiling angles for powder average
%           charpentier.m      -- Expressions from Charpentier's thesis for (static) CT transition frequencies
%           mergeangle_y.m     -- rotates EFG  from PAS to MOL(XTAL) to LAB axes
% other necessary functions
%       DSIM_wigner.m          -- used by DSIM_wconstant to compute Wigner 2nd rank rotation matrices

cq = 2000*pi*param.cq ; % CQ in rad/sec
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
oneangle = param.oneangle;
eta = param.eta;
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
diso = 2*pi*param.w0*param.diso;
NFID = param.nfid;
nppe = param.nppe;
nppe = floor(nppe);
nechoes = param.nechoes;
dwell = param.time*1e-6;
tau = param.pulse*1.e-6; % pulse spacing (seconds)
dink = 2*tau-dwell*floor(nppe);
wl = 2*pi*1.e6*param.w0;        % larmor frequency to rad/sec
dwell = param.time*1e-6;
siten = param.sites_total;
edink = eye(siten);
wts = param.wts(:);             % column vector of relaive weights
magzero = sqrt(wts);  
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
%kmatrix = kmatrix- 2*pi/(NFID*dwell)*eye(size(kmatrix)); % add a little line broadening to stabilize recursion
% powder loop starts here
FID = zeros(1, param.nfid);
FID = zeros(1,2^nextpow2(FID));
for powIndex=1:NAVG
    freqMatrix = zeros(siten);
    for k=1:siten
        PAS = [etasig(k) 0 deltasig(k) 0 etasig(k)];
        LAB = PAS * wigner(oneangle(k,:)) * wigner(Pinfo(powIndex,1:3));
        rot3 = mergeangle_y(oneangle(k,:),Pinfo(powIndex,1:3));
        freqMatrix(k,k) = diso(k)+LAB(1,3);% Chemical shift terms
        freqMatrix(k,k) = freqMatrix(k,k)+charpentier(cq(k),eta(k),wl,param.spin,rot3);
        
        
%         rot1 = oneangle(k,:); % rotation PAS ==> XTAL
%         rot2 = [Pinfo(powIndex,1) Pinfo(powIndex,2) 0]; % rotation XTAL ==> Lab
%         rot3 = mergeangle_y(rot1,rot2); % rotation PAS ==> Lab
%         freqMatrix(k,k) = charpentier(cq(k),eta(k),wl,param.spin,rot3);
    end
    propA = i*freqMatrix + kmatrix ;
    Adwell= expm(propA*dwell);
    Atau1 = expm(conj(propA*tau));
    magz  = Atau1*magzero;
    if dink > 0.001*dwell
        edink = expm(propA*dink);
    end
    FID(1) =  magzero'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
    kk = 0;
    % first echo period
    for timeIndex=1:nppe
        magz = Adwell * magz;
        kk = kk+1;
        FID(timeIndex) = FID(timeIndex) + magzero'*magz * Pinfo(powIndex,4);
    end
    % subsequent echo periods
    magz = edink*magz ; % propagate to end of 2*tau interval if nppe is fractional
    for echonum = 2:nechoes
        magz = conj(magz);
        for timeIndex = 1:nppe
            magz = Adwell * magz;
            FID(kk+timeIndex)= FID(kk+timeIndex)+ magzero'*magz * Pinfo(powIndex,4);
        end
        kk = kk+nppe;
        magz = edink*magz;
    end
end
FID = FID(:);