function FID=DSIMCSA(param)
% FID=DSIMCSA(param)
% Deuteron MAS/OMAS simulation routine
% (includes CSA as well as CQ)
% see the file express.m for parameter definitions
%
% functions called by DSIMCSA
%           gettiles.m         -- get tiling angles for powder average
%           DSIM_wconstant.m   -- used in computing EFG site frequencies in banded Floquet matrix
%           CSA_wconstant.m    -- used in computing CSA site frequencies in banded Floquet matrix
%           makeAmatrix        -- combines rate & site frequencies to make (Floquet) propagator matrix A
%
% other necessary functions
%       DSIM_wigner.m          -- used by DSIM_wconstant & CSA_wconstant to compute Wigner 2nd rank rotation matrices
%       mergeangle_y.m         -- used to express CSA angles in EFG PAS frame

cq = param.cq;
eta = param.eta;
oneangle = param.oneangle;
oneanglecsa = param.oneanglecsa;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrx = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrx = param.custom_onek; % 
    otherwise
        kmatrx = param.onek;
end
actualBeta = 180*acos(sqrt(1/3))/pi;
beta = (param.beta_r+actualBeta)*pi/180; % back to radians...
NFID = param.nfid;
SR = param.sr;
NSIDE = param.pairs;
TIME = param.time*1e-6;
sitenumb = param.sites_total;
cq = 2000*pi*cq;                % cq to Rad/sec
SR=SR*2000*pi;                  % rotation speed to Rad/sec
wts = param.wts(:);             % column vector of relaive weights
magzero = sqrt(wts); % use square roots since A has been symmetrized, even when weights are unequal   
%generate the angle pairs for powder average, ZCW tiling
% CSA parameters
diso = 2*pi*param.w0*param.diso; % isotropic shift in rad/sec
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
% get one-frame angles for CSA tensor in CQ PAS frame
for kk = 1:sitenumb
    rcsa = oneanglecsa(kk,:);  % rotation CSA PAS ==> CQ PAS
    rcq = oneangle(kk,:);      % rotation CQ PAS ==> XTAL
    oneanglecsa(kk,:) = mergeangle_y(rcsa,rcq);
end
jj = 1:sitenumb;
indx = NSIDE*(2*jj-1) + jj;                 % picks out the C(0) coefficient for each site
c1=zeros(2*sitenumb*NSIDE+sitenumb,1);      % the initial C coefficient vector
 
for ii=1:sitenumb                           % create the initial C coeffecients
c1(indx(ii)) = magzero(ii);                       
end      
          
FID=zeros(1,NFID);

for Pindex=1:NAVG-1
    phi=Pinfo(Pindex,1);
    theta=Pinfo(Pindex,2);                                                            
    weight=Pinfo(Pindex,4);     
    wconq=DSIM_wconstant(phi,theta,beta,oneangle,cq,eta);
    wconc = CSA_wconstant(phi,theta,beta,oneanglecsa,diso,deltasig,etasig);
    wplus = wconc+wconq;
    wmin = wconc-wconq;
%     aplus = DSIM_hmatrix(wplus,kmatrx,SR,NSIDE);
%     amin = DSIM_hmatrix(wmin,kmatrx,SR,NSIDE); 
    aplus = makeAmatrix(wplus,kmatrx,NSIDE,SR);
    amin = makeAmatrix(wmin,kmatrx,NSIDE,SR);
    dwp=expm(i*aplus*TIME*SR);
    dwm=expm(i*amin*TIME*SR);
    ctp=c1;
    ctm = c1;
    powdersignal=magzero'*(ctm(indx)+ctp(indx));
    FID(1,1)=FID(1,1)+powdersignal*weight;
    for timeindex=2:NFID                    %begin timeloop
        ctp=dwp*ctp;
        ctm=dwm*ctm;
        powdersignal=magzero'*(ctm(indx)+ctp(indx));
        FID(1,timeindex)=FID(1,timeindex)+powdersignal*weight;
    end
end
FID = FID(:) ; % be sure to return a column vector!
