function FID=T1Zspikelet(param)
% FID=T1Zsim(param)
% Deuteron T1Z simulation routine: T1IR w/QE detection sequence
% (no chemical shifts)
% (ideal, delta-function pulses)
% see the file express.m for parameter definitions
%
% functions called by T1Zsim
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes Wigner 2nd rank rotation matrices
%
% other necessary functions: (none)

cq = param.cq;
eta = param.eta;
NFID = param.nfid;
dwell = param.time*1e-6;
tau1 = param.pulse*1e-6;
wts = param.wts(:); % column vector of relative weights
siten = param.sites_total;
switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
siteAngle = param.oneangle; % for EFG tensors, not CSA...
tau = param.taulist; % relaxation delay list, in seconds
ntau = fix(length(tau));
cq = 2*pi*1000*cq;
wl = 2*pi*param.w0*1.e6 ; % Larmor frequency in rad/sec
eta = eta / sqrt(6); % Divide by sqrt(6) here instead of in the powder loop
nppe = floor(param.nppe);
nechoes = param.nechoes-1;
dink = 2*tau1-dwell*floor(nppe);
edink = eye(siten);
%
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation
%
R2 = 2*pi/(NFID*dwell);
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
% compute factors in spectral densities that are INDEPENDNT of powder angles
% These are JKK'(Mw0) in the xtal fixed frame
[vec,kval] = eig(kmatrix);
lam = -diag(kval);
rat1 = lam./(lam.^2+wl^2);
rat2 = lam./(lam.^2+4*wl^2);
PR2K = zeros(siten,5);
for jj = 1:siten
    PAS = [eta(jj) 0 1 0 eta(jj)]*cq(jj)*sqrt(3/8);
    D2 = wigner(siteAngle(jj,:));
    for k = 1:5
        PR2K(jj,k) = PAS*D2(:,k)*sqrt(wts(jj));
    end
end
J1KKP = zeros(5);
J2KKP = zeros(5);
for b = 1:siten
    if abs(lam(b)) > 0.1 % omit small eigenvalues of K
        for k = 1:5
            for kp = 1:5
                tmp = 0;
                for ii = 1:siten
                    for jj = 1:siten
                        tmp = tmp + PR2K(ii,k)*vec(ii,b)*vec(jj,b)*conj(PR2K(jj,kp));
                    end
                end
                J1KKP(k,kp)=J1KKP(k,kp)+rat1(b)*tmp;
                J2KKP(k,kp)=J2KKP(k,kp)+rat2(b)*tmp;
            end
        end
    end
end

magzero = sqrt(wts)*ones(1,ntau);                 
FID = zeros(NFID, ntau);
for powIndex=1:NAVG
    xtal_wt = Pinfo(powIndex,4);
    D2 = wigner(Pinfo(powIndex,1:3));
    J1 = 0; J2 = 0;
    for k = 1:5
        for kp = 1:5
            J1 = J1 + J1KKP(k,kp)*D2(k,4)*conj(D2(kp,4));
            J2 = J2 + J2KKP(k,kp)*D2(k,5)*conj(D2(kp,5));
        end
    end
    freqMatrix = zeros(siten);
    for k=1:siten
        siteCq  =  cq(k) * 3 / 4;
        siteEta = eta(k);
        PAS = [siteEta 0 1 0 siteEta] * siteCq;
        LAB = PAS * wigner(siteAngle(k,:)) * D2;
        freqMatrix(k,k) = LAB(1,3);
    end
    propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
    Adwell= expm(propA*dwell);
    Atau1 = expm(conj(propA*tau1));
    if dink > 0.001*dwell
        edink = expm(propA*dink);
    end
    for jj = 1:ntau
        mags = magzero(:,jj);
        magz  = Atau1*mags*(1-2*exp(-tau(jj)*real(J1+4*J2)));
        FID(1,jj) =  mags'*magz*xtal_wt; % time origin is just after second pulse
        kk = 1;
        % first echo period
        for timeIndex=1:nppe
            magz = Adwell * magz;
            kk = kk+1;
            FID(timeIndex,jj) = FID(timeIndex,jj) + mags'*magz*xtal_wt;
        end
        magz = edink*magz ; % propagate to end of 2*tau interval if nppe is fractional
        % subsequent echo periods
        for echonum = 2:nechoes
            magz = conj(magz);
            for timeIndex = 1:nppe
                magz = Adwell * magz;
                FID(kk+timeIndex,jj)= FID(kk+timeIndex,jj)+ mags'*magz *xtal_wt;
            end
            kk = kk+nppe;
            magz = edink*magz;
        end
    end
end
FID = real(FID) ;% this symmetrization works only when chemical shifts are all zero


