function FID=staticsimCQCSA(param)
% FID=staticsim(param)
% Deuteron quadrupole echo simulation routine
% (includes CSA but assumes both CQ and CSA are refocussed)
% see the file express.m for parameter definitions
%
% functions called by staticsimCQCSA
%           gettiles.m         -- get tiling angles for powder average
%           wigner.m           -- computes Wigner 2nd rank rotation matrices
%           mergeangle_y.m         -- used to express CSA angles in EFG PAS frame
% other necessary functions: (none)

cq = param.cq;
eta = param.eta;
oneangle = param.oneangle; % for CQ, not CSA
oneanglecsa = param.oneanglecsa;
sitenumb = param.sites_total;

for kk = 1:sitenumb
    rcsa = oneanglecsa(kk,:);  % rotation CSA PAS ==> CQ PAS
    rcq = oneangle(kk,:);      % rotation CQ PAS ==> XTAL
    oneanglecsa(kk,:) = mergeangle_y(rcsa,rcq);
end

switch param.runflags(1) % check for user-modified one-frame K matrix
    case 0 % there is no user-modified, custom one-frame K-matrix
        kmatrix = param.onek; 
    case 1 % use the user-modified one-frame K-matrix
        kmatrix = param.custom_onek; % 
    otherwise
        kmatrix = param.onek;
end
NFID = param.nfid;
dwell = param.time*1e-6;
tau1 = param.pulse*1e-6;
wts = param.wts(:); % column vector of relative weights
siten = param.sites_total;
cq = 2*pi*1000*cq;
eta = eta / sqrt(6); % Divide by sqrt(6) here instead of in the powder loop
diso = 2*pi*param.w0*param.diso; % isotropic shift in rad/sec
% units are rad/sec
deltasig = 2*pi*param.w0*2*param.dsigcsa/3;
etasig = deltasig.*param.etacsa/sqrt(6);
%
% add artificial damping by 2x(digital resolution) to avoid roundoff
% accumulation in recursive FID calculation -- delete the 0* to activate
%
R2 = 0*2*pi/(NFID*dwell);
Pinfo = gettiles(param.n,param.tiletype);
NAVG = size(Pinfo,1);
test = max([abs(diso)  abs(deltasig) abs(etasig)]); % test = 0 implies no CSA   
if abs(test) <= 0.001                    % test = 0 implies no CSA
    magzero = sqrt(wts);                 % sqrt used here because K-matrix has been symmetrized
    FID = zeros(1, NFID);
    for powIndex=1:NAVG
        freqMatrix = zeros(siten);
        for k=1:siten
            siteCq  =  cq(k)*3/4;
            siteEta = eta(k);
            PAS = [siteEta 0 1 0 siteEta]*siteCq;
            LAB = PAS * wigner(oneangle(k,:))*wigner(Pinfo(powIndex,1:3));
            freqMatrix(k,k) = LAB(1,3);
        end
        propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
        Adwell= expm(propA*dwell);
        Atau1 = expm(conj(propA*tau1));
        magz  = Atau1*magzero;
        FID(1) = FID(1) + magzero'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
        for timeIndex=2:NFID-1
            magz = Adwell * magz;
            FID(timeIndex) = FID(timeIndex) + magzero'*magz * Pinfo(powIndex,4);
        end
    end
    FID = FID(:) ;% return a column vector, for compatibility with multi-fid simulation routines
    FID = real(FID); % this symmetrization of the FT spectrum works ONLY for zero chemical shift & CSA!!
    return
else % this branch computes both transitions, including CSA and CQ
    magzero = sqrt(wts);                 % sqrt used here because K-matrix has been symmetrized
    FID = zeros(1, NFID);
    FIDm = FID;
    for powIndex=1:NAVG
        freqMatrix = zeros(siten);
        fm = freqMatrix;
        for k=1:siten
            siteCq  = cq(k)*3/4;
            siteEta = eta(k);
            PAS = [siteEta 0 1 0 siteEta]*siteCq;
            PASsig = [etasig(k) 0 deltasig(k) 0 etasig(k)];
            LAB = PAS*wigner(oneangle(k,:))*wigner(Pinfo(powIndex,1:3));
            LABsig = PASsig*wigner(oneanglecsa(k,:))*wigner(Pinfo(powIndex,1:3));
            freqMatrix(k,k) = LABsig(1,3)+LAB(1,3);
            fm(k,k) = LABsig(1,3)-LAB(1,3);
        end
        propA = i*freqMatrix + kmatrix - R2*eye(size(kmatrix));
        propAm = i*fm + kmatrix - R2*eye(size(kmatrix));
        Adwell= expm(propA*dwell);
        Atau1 = expm(conj(propA*tau1));
        magz  = Atau1*magzero;
        Adwellm= expm(propAm*dwell);
        Atau1m = expm(conj(propAm*tau1));
        magzm  = Atau1m*magzero;
        FID(1) = FID(1) + magzero'*magz*Pinfo(powIndex,4); % time origin is just after second pulse
        FIDm(1) = FIDm(1) + magzero'*magzm*Pinfo(powIndex,4);
        for timeIndex=2:NFID-1
            magz = Adwell*magz;
            FID(timeIndex) = FID(timeIndex) + magzero'*magz*Pinfo(powIndex,4);
            magzm = Adwellm*magzm;
            FIDm(timeIndex) = FIDm(timeIndex) + magzero'*magzm*Pinfo(powIndex,4);            
        end
    end
    FID = FID(:)+FIDm(:) ;% return a column vector, for compatibility with multi-fid simulation routines
    return
end