function varargout = mpr(varargin)
%
% processes (multiple) fid signals generated by EXPRESS
% 
% Performs left shift, phasing, apodization zero fill, and Fourier transformation
% using spectral parameters saved from the original EXPRESS simulation.
%
% Usage:
%
%          spec = mpr(param,fids)       -- returns processed spectra corresponding to parameters in param
%    [spec,fax] = mpr(param,fids)       -- returns spectra and a commensurate frequency axis (units are kHz)
%    [spec,fax] = mpr(param,fids,units) -- returns spectra and frequency axis in kHz, Hz or ppm
%                                          where units is a string:  'Hz', 'kHz', or 'ppm'  (quotes required)
%
% Parameters used by this routine include:
%  
% param.ls   -- number of left shifts (can be fractional)
% param.lb   -- Lorentzian line broadening (input in kHz, NOT Hz!)
% param.gb   -- gaussian line broadening (input in kHz, NOT Hz!)
% param.sw   -- spectral width (1/DW) (input in kHz) Frequency axis will be -sw/2 to ~ +sw/2
% param.npts -- final number of points, after zero fill ( not necessarily the original number of points simulated)
% param.experiment -- invokes special processing for simulated T1q signals (experiment = '2H T1Q')
% param.w0   -- Larmor frequency (MHz) used to generate frequency axis in user-defined units
% 

switch nargin % checks number of input arguments
    case 2     
        param = varargin{1};
        sig = varargin{2};
        units = 'kHz' ; % default units for frequency axis
    case 3      
        param = varargin{1};
        sig = varargin{2};
        units = varargin{3};
    otherwise
        errordlg( 'Type help mpr for assistance with using this routine')
        return
end
ls = param.left_shift;      % number of left shifts (can be fractional)
lb = 1000*param.lb ;        % Lorenztian line broadening (Hz)
gb = 1000*param.gb ;        % Gaussina linebroadening, Hz
sw = 1000*param.swh;        % spectral window, Hertz
npts = param.zfpts;         % number of points with current zero fill, NOT necessarily the original number of points
switch upper(units)         % units (case insensitive) for frequency axis
    case 'KHZ'
        ufac = 1.0 ;        
    case 'HZ'
    ufac = 1000;
    case 'PPM'
        ufac = 1000/param.w0;
    otherwise
        ufac = 1.0 ;
end
fax = ufac*([0:(npts-1)]'-npts/2)*sw/npts/1000; % frequency axis (kHz) in appropriate units
[n , ntau] = size(sig);     % n is the original number of points; ntau is the number of tau values
ph0 = 0;                    % zero order phase
ph1 = 0;                    % first order phase
tt = [0:(n-1)]'/sw;                             % units of tt are seconds; sw is Hertz
R = pi*lb;                                      % EM
G = sign(gb)*((pi*gb/(2))^2)/log(2);            % GM
apod = exp(-R*tt).*exp(-G*tt.*tt)*ones(1,ntau); % apodization function
nt = size(apod,1);

buff = zeros(nt+npts-n,ntau);
spec = zeros(size(buff)) ;                % a simple way to allocate space for spec
for jj = 1: size(sig,2)
    buf = apod(:,jj).*sig(:,jj);
    % get last point(s) for DC offset correction of one-sided DFT
    last = buf(n,:);
    % zero order phase correct in time domain,then left/right shift
    buf = phcor(buf,ph0);
    buf = shift2d(buf,ls);
    % zero fill
    
    % FT and linear phase correction
    buf(1,:) = (buf(1,:)+last)/2 ; % removes DC offset from one-sided DFT
    if (npts-n) > 0
    buff(:,jj) = [buf ;zeros(npts-n,1)];
    else
        buff = buf;
    end
    buff =  phcor(swap(fft(buff)),0,ph1);
    n2 = floor(size(buff,1)/2); % first n/2 points
    switch param.experiment
        case '2H T1Q'
            buff(1:n2 ,:) = -buff(1:n2,:); % invert one side of spectrum
            buff = (spec-flipud(buff))/2;  % add the other transition (assumes ideal pulses,etc)
            spec(:,jj) = real(buff(:,jj));
        otherwise
            spec(:,jj) = real(buff(:,jj));
    end
    spec(:,jj) = real(buff(:,jj));
end
switch nargout
    case 1
        varargout{1} = spec;
    case 2
        varargout{1} = spec;
        varargout{2} = fax;
    otherwise
        varargout{1} = spec;
end
    