function newparam = rebuild(param,opt)
%
%   newparam = rebuild(param,val,opt)
%
%   This routine returns an EXPRESS-compatible parameter structure in which
%   one (vector) parameter has been changed. 
%
%   param:   is the input parameter structure, which must include the NEW
%            parameter values that require updates of one-frame equivalents.
%   opt:     must be 'rate' or 'CQangles' or 'CSAangles'
%
%   This routine rebuilds the one-frame rate matrix (opt = 'rate') or the
%   list of one-frame site angles (opt = 'angles'). It is not needed and
%   should not be used if neither the one-frame rate matrix nor the
%   one-frame angles need to be updated. Note that changing unequal site
%   populations DOES change the one-frame (symmetrized) rate matrix.
% 

nframes = param.nframes;
newparam = param; % copy the input parameter structure
for n=1:nframes
    Nsite(n) = param.frames{n}.sites;
    Nmodel(n,:) = param.frames{n}.model;
    Nrate(n) = param.frames{n}.rate;
    Ntemplate{n} = param.frames{n}.template;
end
switch opt
    case 'rates'
        [onek, kmatrxcell]=oneframekmatrx(Nsite,Nmodel, Nrate, Ntemplate); % re-build the one-frame kmatrix
        siten = size(onek,1);
        for jj = 1:siten % symmetrize the rate matrix for use with unequal populations
            tmp = 0;
            for kk = 1:siten
                if kk ~= jj
                    tmp = tmp -2*param.wts(kk)*onek(jj,kk)/(param.wts(jj)+param.wts(kk));
                end
            end
            onek(jj,jj) = tmp;
        end
        for jj = 1:siten
            for kk = 1:siten
                onek(jj,kk) = 2*onek(jj,kk)*sqrt(param.wts(jj)*param.wts(kk))/(param.wts(jj)+param.wts(kk));
            end
        end
        newparam.onek = onek;
        [vec,val] = eig(param.onek); % diagonalize K-matrix for use in spin lattice calculations
        newparam.onekvec = vec;
        newparam.onekval = diag(val);
        clear vec val;
        newparam.kmatrxcell = kmatrxcell;
    case 'CQangles'
        Fangle = zeros(sum(Nsite),3);
        sitenum=1;
        for n=1:nframes
            for siteadd=1:Nsite(n)
                angle2 = [pi*param.frames{n}.cq_phi(siteadd)/180 pi*param.frames{n}.cq_theta(siteadd)/180 pi*param.frames{n}.cq_psi(siteadd)/180];
                Fangle(siteadd+sitenum-1,:) = angle2;
            end
            sitenum = sitenum + Nsite(n);
        end
        [oneangle Tsite]=oneframeangle(Nsite,Fangle);
        newparam.oneangle = oneangle;
    case 'CSAangles'
        Fanglecsa = zeros(sum(Nsite),3);
        sitenum=1;
        for n=1:nframes
            for siteadd=1:Nsite(n)
                angle1 = [pi*param.frames{n}.csa_phi(siteadd)/180 pi*param.frames{n}.csa_theta(siteadd)/180 pi*param.frames{n}.csa_psi(siteadd)/180];
                Fanglecsa(siteadd+sitenum-1,:) = angle1;
            end
            sitenum = sitenum + Nsite(n);
        end
        [oneanglecsa Tsite]=oneframeangle(Nsite,Fanglecsa);
        newparam.oneanglecsa = oneanglecsa;
end
